import { QueryBuilderState, SchemaState } from '../store/querybuilderSlice';
import { URLParams, setParam } from '../api/url';
import { Broker } from './broker';
import { DateStringStatement } from '../../querybuilder/model/logic/general';
import { VisState } from '../store/visualizationSlice';
import { AuthorizationOperations } from '../store/authSlice';

export const databaseNameMapping: string[] = ['arangodb', 'neo4j'];
export const databaseProtocolMapping: string[] = ['neo4j://', 'neo4j+s://', 'bolt://', 'bolt+s://'];

export enum DatabaseType {
  ArangoDB = 'arango',
  Neo4j = 'neo4j',
}

export enum DatabaseStatus {
  untested = 0,
  testing = 1,
  tested = 2,
}

export type DatabaseInfo = {
  internalDatabaseName: string;
  url: string;
  port: number;
  protocol: string;
  username: string;
  password: string;
  type: string;
};

export const SaveStateAuthorizationObjectsArray = ['database', 'visualization', 'query', 'schema'] as const;
export type SaveStateAuthorizationObjects = (typeof SaveStateAuthorizationObjectsArray)[number];

export type SaveStateAuthorizationHeaders = {
  [id in SaveStateAuthorizationObjects]: {
    [id in AuthorizationOperations]: boolean;
  };
};

export const nilUUID = '00000000-0000-0000-0000-000000000000';

export type SaveStateI = {
  id: string;
  userId: string;
  name: string;
  dbConnections: DatabaseInfo[];
  schemas: SchemaState[];
  queries: QueryBuilderState[];
  visualizations: VisState;
  createdAt: string;
  updatedAt: string;
  shareState: any;
};

type GetStateResponse = (data: SaveStateI, status: string) => void;
export function wsGetState(saveStateID: string, callback?: GetStateResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'get',
      body: { saveStateID: saveStateID }, //messageTypeGetSaveState
    },
    callback,
  );
}
export function wsGetStateSubscription(callback: GetStateResponse) {
  const id = Broker.instance().subscribe(callback, 'save_state');
  return () => {
    Broker.instance().unSubscribe('save_state', id);
  };
}

type GetStatesResponse = (data: SaveStateI[], status: string) => void | boolean;
export function wsGetStates(callback?: GetStatesResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'getAll',
    },
    callback,
  );
}
export function wsGetStatesSubscription(callback: GetStatesResponse) {
  const id = Broker.instance().subscribe((data: any, status: string) => {
    if (data && status) {
      callback(data, status);
    }
  }, 'save_states');
  return () => {
    Broker.instance().unSubscribe('save_states', id);
  };
}

export function wsCreateState(request: SaveStateI, callback?: GetStateResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'create',
      body: request, //SaveStateModel
    },
    callback,
  );
  // Also returns save_state
}

type DeleteStateResponse = (status: 'deleted' | DateStringStatement) => void;
export function wsDeleteState(id: string, callback?: Function) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'delete',
      body: { saveStateID: id }, //messageTypeGetSaveState
    },
    callback,
  );
}
export function wsDeleteStateSubscription(callback: DeleteStateResponse) {
  const id = Broker.instance().subscribe(callback, 'delete_save_state');
  return () => {
    Broker.instance().unSubscribe('delete_save_state', id);
  };
}

type TestSaveStateConnectionResponse = (data: { status: 'success' | 'fail'; saveStateID?: string }) => void;
export function wsTestSaveStateConnection(id: string, callback?: Function) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'testConnection',
      body: { saveStateID: id }, //messageTypeGetSaveState
    },
    callback,
  );
}
export function wsTestSaveStateConnectionSubscription(callback: TestSaveStateConnectionResponse) {
  const id = Broker.instance().subscribe(callback, 'tested_connection');
  return () => {
    Broker.instance().unSubscribe('tested_connection', id);
  };
}

export function wsUpdateState(request: SaveStateI, callback?: GetStateResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'update',
      body: request, //SaveStateModel
    },
    callback,
  );
  // Also returns save_state
}

export function wsTestDatabaseConnection(dbConnection: DatabaseInfo, callback?: TestSaveStateConnectionResponse) {
  if (!dbConnection) {
    console.warn('dbConnection is undefined on wsTestDatabaseConnection');
    if (callback) callback({ status: 'fail', saveStateID: '' });
    return;
  }
  Broker.instance().sendMessage(
    {
      key: 'dbConnection',
      subKey: 'testConnection',
      body: dbConnection, //DBConnectionModel
    },
    callback,
  );
}

type StateGetPolicyResponse = (data: SaveStateAuthorizationHeaders) => void;
export function wsStateGetPolicy(saveStateID: string, callback?: StateGetPolicyResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'getPolicy',
      body: { saveStateID: saveStateID },
    },
    callback,
  );
}

type StateSetPolicyRequest = {
  saveStateId: string;
  users: {
    userId: string;
    sharing: SaveStateAuthorizationHeaders;
  }[];
};
type ShareStateResponse = (data: boolean) => void;
export function wsShareSaveState(request: StateSetPolicyRequest, callback?: ShareStateResponse) {
  Broker.instance().sendMessage(
    {
      key: 'state',
      subKey: 'shareState',
      body: request,
    },
    callback,
  );
}
export function wsShareStateSubscription(callback: ShareStateResponse) {
  const id = Broker.instance().subscribe((data: any) => {
    if (data) {
      callback(data);
    }
  }, 'share_save_state');
  return () => {
    Broker.instance().unSubscribe('share_save_state', id);
  };
}
