import { useEffect, useMemo, useState } from 'react';
import { Tooltip, TooltipTrigger, TooltipContent, DropdownItem, TextInput, Icon } from '../../components';
import { ReactFlowInstance, Node } from 'reactflow';
import { NodeAttribute, SchemaReactflowEntityNode } from '../model';
import {
  useAppDispatch,
  useQuerybuilderAttributesShown,
  useQuerybuilderGraph,
  useQuerybuilderHash,
  useQuerybuilderSettings
} from '../..';
import { isEqual } from 'lodash-es';
import {
  attributeShownToggle,
  QueryUnionType,
  setQuerybuilderGraphology,
  setQueryUnionType,
  toQuerybuilderGraphology,
} from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import { getDataTypeIcon } from '../../components/DataTypeIcon';

export const ContextMenu = (props: {
  open: boolean;
  position?: { x: number; y: number };
  node?: Node;
  reactFlowWrapper: React.RefObject<HTMLDivElement>;
  reactFlow: ReactFlowInstance;
  onClose: () => void;
}) => {
  const [filter, setFilter] = useState<string>('');
  const dispatch = useAppDispatch();
  const graph = useQuerybuilderGraph();
  const qbHash = useQuerybuilderHash();
  const settings = useQuerybuilderSettings();
  const unionType = settings.unionTypes == null || props.node == null ? QueryUnionType.AND : settings.unionTypes[props.node.id];

  const graphologyGraph = useMemo(() => toQuerybuilderGraphology(graph), [graph, qbHash]);

  const state = useMemo(() => {
    const divPos = props.reactFlowWrapper.current?.getBoundingClientRect();
    if (!divPos || !props.node) return;
    let position = { x: 0, y: 0 };
    if (props.position) {
      position = props.position;
    } else {
      position = props.reactFlow.flowToScreenPosition({ x: props.node.data.x, y: props.node.data.y });
    }

    return {
      open: props.open,
      x: position.x - divPos.x,
      y: position.y - divPos.y + 10,
    };
  }, [props.open, props.node]);

  const filteredAttributes = useMemo<NodeAttribute[]>(() => {
    if (props.node == null) return [];
    if (filter == null || filter.length == 0) return props.node.data.attributes;

    return (props.node.data.attributes as NodeAttribute[]).filter((attr) => {
      return attr.handleData.attributeName?.toLocaleLowerCase().includes(filter.toLocaleLowerCase());
    });
  }, [filter, props.node]);

  const attributesBeingShown = useQuerybuilderAttributesShown();
  function isAttributeAdded(attribute: NodeAttribute): boolean {
    return attributesBeingShown.some((x) => isEqual(x, attribute.handleData));
  }

  function addAttribute(attribute: NodeAttribute) {
    dispatch(attributeShownToggle(attribute.handleData));
  }

  function setUnionType(unionType: QueryUnionType) {
    if (!props.node) return;
    dispatch(setQueryUnionType({ nodeId: props.node.id, unionType: unionType }));
  }
  function removeNode() {
    if (!props.node) return;
    const connectedLogicPills = graphologyGraph.neighbors(props.node.id);
    connectedLogicPills.forEach((pill) => {
      const attributes = graphologyGraph.getNodeAttributes(pill);
      if (attributes.type === 'logic') {
        graphologyGraph.dropNode(pill);
      }
    });

    graphologyGraph.dropNode(props.node.id);
    dispatch(setQuerybuilderGraphology(graphologyGraph));
    props.onClose();
  }

  return (
    <Tooltip open={props.open && state !== undefined} interactive={true} showArrow={false} placement="bottom-start">
      <TooltipTrigger x={state ? state.x : 0} y={state ? state.y : 0} />
      <TooltipContent>
        <DropdownItem
          value={'Add/remove attribute'}
          onClick={(e) => {}}
          submenu={[
            <TextInput
              type={'text'}
              placeholder="Filter"
              size="xs"
              className="mb-1 min-w-40 rounded-sm"
              value={filter}
              onClick={(e) => e.stopPropagation()}
              onChange={(v) => setFilter(v)}
            />,

            filteredAttributes.map((attr) => (
              <DropdownItem
                key={attr.handleData.attributeName + attr.handleData.nodeId}
                value={attr.handleData.attributeName ?? ''}
                selected={isAttributeAdded(attr)}
                onClick={(_) => addAttribute(attr)}
              >
                <Icon component={getDataTypeIcon(attr?.handleData?.attributeType)} className="ms-2 float-end" size={16} />
              </DropdownItem>
            )),
          ]}
        />
        <DropdownItem
          value="Union type"
          submenu={[
            <DropdownItem
              value="AND"
              onClick={(_) => setUnionType(QueryUnionType.AND)}
              selected={props.node ? unionType != QueryUnionType.OR : false} // Also selected when null
            />,
            <DropdownItem
              value="OR"
              onClick={(_) => setUnionType(QueryUnionType.OR)}
              selected={props.node ? unionType == QueryUnionType.OR : false}
            />,
          ]}
        />
        <DropdownItem value="Remove" className="text-danger" onClick={(e) => removeNode()} />
      </TooltipContent>
    </Tooltip>
  );
};
